@@ -29,8 +29,17 @@ martCheck = function(mart, biomart = NULL){
     stop("You must provide a valid Mart object. To create a Mart object use the function: useMart.  Check ?useMart for more information.")
   }
   if(!is.null(biomart)){
-    martcheck = strsplit(martBM(mart),"_", fixed = TRUE, useBytes = TRUE)[[1]][1]
-    if(martcheck[1] != biomart)stop(paste("This function only works when used with the ",biomart," BioMart.",sep="")) 
+ #   martcheck = strsplit(martBM(mart),"_", fixed = TRUE, useBytes = TRUE)[[1]][1]
+    martcheck = martBM(mart)
+    bmok = FALSE
+    for(k in 1:length(biomart)){
+    if(martcheck[1] == biomart[k]){	
+     bmok = TRUE
+    }
+    }		    
+    if(!bmok){
+	stop(paste("This function only works when used with the ",biomart," BioMart.",sep="")) 
+    }      
   }
   if(martDataset(mart)==""){
     stop("No dataset selected, please select a dataset first.  You can see the available datasets by using the listDatasets function see ?listDatasets for more information.  Then you should create the Mart object by using the useMart function.  See ?useMart for more information");
@@ -55,16 +64,27 @@ bmRequest <- function(request, ssl.verifypeer = TRUE, verbose = FALSE){
   return(result)
 }
 
-listMarts <- function( mart, host="www.biomart.org", path="/biomart/martservice", port=80,includeHosts = FALSE, archive = FALSE, ssl.verifypeer = TRUE, verbose = FALSE){
-
-  if(archive){
-    request = paste("http://",host,":",port,path,"?type=registry_archive&requestid=biomaRt", sep="")
-    registry = bmRequest(request = request, ssl.verifypeer = ssl.verifypeer, verbose = verbose)
+listMarts <- function( mart = NULL, host="www.biomart.org", path="/biomart/martservice", port=80,includeHosts = FALSE, archive = FALSE, ssl.verifypeer = TRUE, verbose = FALSE){
+  
+  request = NULL
+  if(is.null(mart)){	  
+   if(archive){
+      request = paste("http://",host,":",port,path,"?type=registry_archive&requestid=biomaRt", sep="")
+   } 
+   else{
+      request = paste("http://",host,":",port,path,"?type=registry&requestid=biomaRt", sep="")	
+    }
   }
   else{
-    request = paste("http://",host,":",port,path,"?type=registry&requestid=biomaRt", sep="")	
-    registry = bmRequest(request = request, ssl.verifypeer = ssl.verifypeer, verbose = verbose)
-  }
+     if(class(mart) == 'Mart'){
+         request = paste(martHost(mart),"?type=registry&requestid=biomaRt", sep="") 
+     }
+     else{
+	warning(paste(mart,"object needs to be of class Mart created with the useMart function.  If you don't have a Mart object yet, use listMarts without arguments or only specify the host argument",sep=" "))
+     }
+  } 	
+  
+  registry = bmRequest(request = request, ssl.verifypeer = ssl.verifypeer, verbose = verbose)
   registry = xmlTreeParse(registry, asText=TRUE)
   registry = registry$doc$children[[1]]
   
@@ -75,14 +95,14 @@ listMarts <- function( mart, host="www.biomart.org", path="/biomart/martservice"
     for(i in seq(len=xmlSize(registry))){
       if(xmlName(registry[[i]])=="MartURLLocation"){  
         if(xmlGetAttr(registry[[i]],"visible") == 1){
-          if(!is.null(xmlGetAttr(registry[[i]],"name"))) marts$biomart[index] = xmlGetAttr(registry[[i]],"name")
-          if(!is.null(xmlGetAttr(registry[[i]],"database"))) marts$database[index] = xmlGetAttr(registry[[i]],"database")
-          if(!is.null(xmlGetAttr(registry[[i]],"displayName"))) marts$version[index] = xmlGetAttr(registry[[i]],"displayName")
-          if(!is.null(xmlGetAttr(registry[[i]],"host"))) marts$host[index] = xmlGetAttr(registry[[i]],"host")
-          if(!is.null(xmlGetAttr(registry[[i]],"path"))) marts$path[index] = xmlGetAttr(registry[[i]],"path")
-          if(!is.null(xmlGetAttr(registry[[i]],"port"))) marts$port[index] = xmlGetAttr(registry[[i]],"port")
+          if(!is.null(xmlGetAttr(registry[[i]],"name"))) marts$biomart[index] = as.character(xmlGetAttr(registry[[i]],"name"))
+          if(!is.null(xmlGetAttr(registry[[i]],"database"))) marts$database[index] = as.character(xmlGetAttr(registry[[i]],"database"))
+          if(!is.null(xmlGetAttr(registry[[i]],"displayName"))) marts$version[index] = as.character(xmlGetAttr(registry[[i]],"displayName"))
+          if(!is.null(xmlGetAttr(registry[[i]],"host"))) marts$host[index] = as.character(xmlGetAttr(registry[[i]],"host"))
+          if(!is.null(xmlGetAttr(registry[[i]],"path"))) marts$path[index] = as.character(xmlGetAttr(registry[[i]],"path"))
+          if(!is.null(xmlGetAttr(registry[[i]],"port"))) marts$port[index] = as.character(xmlGetAttr(registry[[i]],"port"))
           if(!is.null(xmlGetAttr(registry[[i]],"serverVirtualSchema"))){
-            marts$vschema[index] =  xmlGetAttr(registry[[i]],"serverVirtualSchema")
+            marts$vschema[index] =  as.character(xmlGetAttr(registry[[i]],"serverVirtualSchema"))
           }
           index=index+1
         }
@@ -112,12 +132,11 @@ listMarts <- function( mart, host="www.biomart.org", path="/biomart/martservice"
   }
   else{
     if(archive){
-      ret = data.frame(marts$database,marts$version)
+      ret = data.frame(biomart = as.character(marts$database),version = as.character(marts$version), stringsAsFactors=FALSE)
     }
     else{
-      ret = data.frame(marts$biomart,marts$version)
+      ret = data.frame(biomart = as.character(marts$biomart),version = as.character(marts$version), stringsAsFactors=FALSE)
     }
-    colnames(ret)=c("biomart","version")
     return(ret)
   } 
 }
@@ -668,13 +687,26 @@ getLDS <- function(attributes, filters = "", values = "", mart, attributesL, fil
     cat(paste(xmlQuery,"\n", sep=""))
   }
   postRes = postForm(paste(martHost(mart),"?",sep=""),"query"=xmlQuery)
-  
+  ## 10-01-2014
+  if(length(grep("^Query ERROR", postRes))>0L)
+     stop(postRes)  
+  ##
   if(postRes != ""){
     con = textConnection(postRes)
     result = read.table(con, sep="\t", header=bmHeader, quote = "\"", comment.char = "", as.is=TRUE, check.names = TRUE)
     close(con)
     if(all(is.na(result[,ncol(result)])))
       result = result[,-ncol(result),drop=FALSE]
+## 10 - 01 - 2014
+   res_attributes <- c(attributes,attributesL)
+      if(!(is(result, "data.frame") && (ncol(result)==length(res_attributes)))) {
+         print(head(result))
+         stop("The query to the BioMart webservice returned an invalid result: the number of columns in the result table does not equal the number of attributes in the query. Please report this to the mailing list.")
+     } 
+    if(!bmHeader){  #assumes order of results same as order of attibutes in input
+     colnames(result) = res_attributes
+##
+   }
   } else {
     warning("getLDS returns NULL.")
     result=NULL
@@ -732,6 +764,70 @@ getBMlist <- function(attributes, filters = "", values = "", mart, list.names =
 #Ensembl specific functions   #
 ###############################
 
+listEnsembl <- function(mart = NULL, host="www.ensembl.org",version = NULL, GRCh = NULL,verbose = FALSE){
+ if(!is.null(version)){
+  host = paste("e",version,".ensembl.org",sep="")
+ }
+ if(!is.null(GRCh)){
+  if(GRCh == 37){ 
+  host = paste("grch",GRCh,".ensembl.org",sep="")
+ }
+else{
+ print("Only 37 can be specified for GRCh version")
+}
+ }
+	   
+ marts = listMarts(mart = mart, host = host, verbose = verbose)
+ sel = which(marts$biomart == "ENSEMBL_MART_ENSEMBL")
+ if(length(sel) > 0){ 
+  marts$biomart[sel] = "ensembl"
+ }
+ sel = which(marts$biomart == "ENSEMBL_MART_SNP")
+ if(length(sel) > 0){ 
+  marts$biomart[sel] = "snp"
+ }
+ sel = which(marts$biomart == "ENSEMBL_MART_FUNCGEN")
+ if(length(sel) > 0){ 
+  marts$biomart[sel] = "regulation"
+ }
+ sel = which(marts$biomart == "ENSEMBL_MART_VEGA")
+ if(length(sel) > 0){ 
+  marts$biomart[sel] = "vega"
+ }
+ return(marts)
+}
+
+useEnsembl <- function(biomart, dataset,host = "www.ensembl.org", version = NULL, GRCh = NULL, verbose = FALSE){
+ 
+ if(!is.null(version)){
+  host = paste("e",version,".ensembl.org",sep="")
+ }	   
+ if(!is.null(GRCh)){
+ if(GRCh == 37){
+  host = paste("grch",GRCh,".ensembl.org",sep="")
+ }
+ else{
+   print("Only 37 can be specified for GRCh version")
+ }
+ }
+
+ if(biomart == "ensembl"){
+   biomart = "ENSEMBL_MART_ENSEMBL"
+ }
+ if(biomart == "snp"){
+   biomart = "ENSEMBL_MART_SNP"
+ }
+ if(biomart == "regulation"){
+   biomart = "ENSEMBL_MART_FUNCGEN"
+ }
+ if(biomart == "vega"){
+   biomart = "ENSEMBL_MART_VEGA"
+ }
+
+ ens = useMart(biomart = biomart, dataset = dataset, host = host, verbose = verbose)	   
+ return(ens)
+}
+
 getGene <- function( id, type, mart){
   martCheck(mart,"ensembl") 
   checkWrapperArgs(id, type, mart)
@@ -743,7 +839,7 @@ getGene <- function( id, type, mart){
 }
 
 getSequence <- function(chromosome, start, end, id, type, seqType, upstream, downstream, mart, verbose=FALSE){
-  martCheck(mart,"ensembl")
+  martCheck(mart,c("ensembl","ENSEMBL_MART_ENSEMBL"))
   if(missing(seqType) || !seqType %in% c("cdna","peptide","3utr","5utr", "gene_exon", "transcript_exon","transcript_exon_intron","gene_exon_intron","coding","coding_transcript_flank","coding_gene_flank","transcript_flank","gene_flank")){
     stop("Please specify the type of sequence that needs to be retrieved when using biomaRt in web service mode.  Choose either gene_exon, transcript_exon,transcript_exon_intron, gene_exon_intron, cdna, coding,coding_transcript_flank,coding_gene_flank,transcript_flank,gene_flank,peptide, 3utr or 5utr")
   }
